/*! asset-hash v3.1.0 by Sebastian Werner <s.werner@sebastian-software.de> */
'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

require('core-js/modules/es.array.slice.js');
require('core-js/modules/es.date.to-string.js');
require('core-js/modules/es.object.to-string.js');
require('core-js/modules/es.parse-int.js');
require('core-js/modules/es.promise.js');
require('core-js/modules/es.regexp.exec.js');
require('core-js/modules/es.regexp.to-string.js');
var crypto = require('crypto');
var fs = require('fs');
var path = require('path');
var BigInt = require('big.js');
var metrohash = require('metrohash');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e['default'] : e; }

var BigInt__default = /*#__PURE__*/_interopDefaultLegacy(BigInt);

var xxhash = null;

try {
  xxhash = require("xxhash");
} catch (_unused) {}

var XXHash32 = xxhash ? xxhash : null,
    XXHash64 = xxhash ? xxhash.XXHash64 : null,
    DEFAULT_HASH = "metrohash128",
    DEFAULT_ENCODING = "base52",
    DEFAULT_MAX_LENGTH = 8,
    XXHASH_CONSTRUCT = 0xcafebabe,
    baseEncodeTables = {
  26: "abcdefghijklmnopqrstuvwxyz",
  32: "123456789abcdefghjkmnpqrstuvwxyz",
  36: "0123456789abcdefghijklmnopqrstuvwxyz",
  49: "abcdefghijkmnopqrstuvwxyzABCDEFGHJKLMNPQRSTUVWXYZ",
  52: "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ",
  58: "123456789abcdefghijkmnopqrstuvwxyzABCDEFGHJKLMNPQRSTUVWXYZ",
  62: "0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ"
},
    BYTE_SIZE = 256;
function baseEncode(buffer, base) {
  var baseNum = typeof base == "number" ? base : parseInt(/\d+/.exec(base)[0], 10),
      encodeTable = baseEncodeTables[baseNum];

  if (!encodeTable) {
    throw new Error("Unknown base encoding " + base + "!");
  }

  var length = buffer.length;
  BigInt__default.DP = 0;
  BigInt__default.RM = 0;

  for (var current = new BigInt__default(0), i = length - 1; i >= 0; i--) {
    current = current.times(BYTE_SIZE).plus(buffer[i]);
  }

  var output = "";

  while (current.gt(0)) {
    output = "" + encodeTable[current.mod(baseNum).toNumber()] + output;
    current = current.div(baseNum);
  }

  BigInt__default.DP = 20;
  BigInt__default.RM = 1;
  return output;
}

function computeDigest(bufferOrString, _temp) {
  var _ref = _temp === void 0 ? {} : _temp,
      encoding = _ref.encoding,
      maxLength = _ref.maxLength,
      output = "";

  if (typeof bufferOrString == "string" && encoding === "hex") {
    output = bufferOrString;
  } else {
    var buffer = typeof bufferOrString == "string" ? Buffer.from(bufferOrString, "hex") : bufferOrString;

    if (encoding === "hex" || encoding === "base64" || encoding === "utf8") {
      output = buffer.toString(encoding);
    } else {
      output = baseEncode(buffer, encoding);
    }
  }

  return maxLength == null || output.length <= maxLength ? output : output.slice(0, maxLength);
}

var Hasher = /*#__PURE__*/function () {
  function Hasher(options) {
    var _options$hash, _options$encoding, _options$maxLength;

    if (options === void 0) {
      options = {};
    }

    this.hasher = void 0;
    this.encoding = void 0;
    this.maxLength = void 0;
    this.hasher = createHasher((_options$hash = options.hash) != null ? _options$hash : DEFAULT_HASH);
    this.encoding = ((_options$encoding = options.encoding) != null ? _options$encoding : DEFAULT_ENCODING) + "";
    this.maxLength = (_options$maxLength = options.maxLength) != null ? _options$maxLength : null;
  }

  var _proto = Hasher.prototype;

  _proto.update = function update(data) {
    var buffer = data instanceof Buffer ? data : Buffer.from(data.toString(), "utf-8");
    return this.hasher.update(buffer);
  };

  _proto.digest = function digest(encoding, maxLength) {
    return computeDigest(this.hasher.digest(), {
      encoding: encoding != null ? encoding : this.encoding,
      maxLength: maxLength != null ? maxLength : this.maxLength
    });
  };

  return Hasher;
}();

function xxhashEnvelope(xxhash) {
  return {
    update: function update(input) {
      return xxhash.update(input);
    },
    digest: function digest() {
      return xxhash.digest("buffer");
    }
  };
}

function cryptoEnvelope(hash) {
  var envelopeHash = {
    update: function update(input) {
      hash.update(input);
      return envelopeHash;
    },
    digest: function digest() {
      return hash.digest();
    }
  };
  return envelopeHash;
}

function createHasher(hash) {
  var hasher;

  if (hash === "xxhash32") {
    if (!XXHash32) {
      throw new Error("Install xxhash module to use xxhash32 hasher");
    }

    hasher = xxhashEnvelope(new XXHash32(XXHASH_CONSTRUCT));
  } else if (hash === "xxhash64") {
    if (!XXHash64) {
      throw new Error("Install xxhash module to use xxhash64 hasher");
    }

    hasher = xxhashEnvelope(new XXHash64(XXHASH_CONSTRUCT));
  } else if (hash === "metrohash64") {
    hasher = new metrohash.MetroHash64();
  } else if (hash === "metrohash128") {
    hasher = new metrohash.MetroHash128();
  } else {
    hasher = cryptoEnvelope(crypto.createHash(hash));
  }

  return hasher;
}
function getHash(fileName, options) {
  var _ref2 = options || {},
      hash = _ref2.hash,
      encoding = _ref2.encoding,
      maxLength = _ref2.maxLength;

  return new Promise(function (resolve, reject) {
    try {
      var hasher = createHasher(hash || DEFAULT_HASH);
      fs.createReadStream(fileName).on("data", function (data) {
        hasher.update(data);
      }).on("error", function (error) {
        reject(error);
      }).on("end", function () {
        try {
          var _digest = computeDigest(hasher.digest(), {
            encoding: encoding || DEFAULT_ENCODING,
            maxLength: maxLength || DEFAULT_MAX_LENGTH
          });

          resolve(_digest);
        } catch (error) {
          reject(error);
        }
      });
    } catch (error) {
      reject(error);
    }
  });
}
function getHashedName(fileName, options) {
  return new Promise(function ($return, $error) {
    var hashed, extension;
    return Promise.resolve(getHash(fileName, options)).then(function ($await_4) {
      try {
        hashed = $await_4;
        extension = path.extname(fileName);
        return $return(hashed + extension);
      } catch ($boundEx) {
        return $error($boundEx);
      }
    }, $error);
  });
}

exports.Hasher = Hasher;
exports.baseEncode = baseEncode;
exports.createHasher = createHasher;
exports.getHash = getHash;
exports.getHashedName = getHashedName;
//# sourceMappingURL=index.cjs.js.map
